/*
Copyright (c) 2000-2003, Jelle Kok, University of Amsterdam
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of the University of Amsterdam nor the names of its
contributors may be used to endorse or promote products derived from this
software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*! \file PlayerTeams.cpp
<pre>
<b>File:</b>          PlayerTest.cpp
<b>Project:</b>       Robocup Soccer Simulation Team: UvA Trilearn
<b>Authors:</b>       Jelle Kok
<b>Created:</b>       10/12/2000
<b>Last Revision:</b> $ID$
<b>Contents:</b>      This file contains the class definitions for the
                      Player that are used to test the teams' high level
                      strategy.
<hr size=2>
<h2><b>Changes</b></h2>
<b>Date</b>             <b>Author</b>          <b>Comment</b>
10/12/2000        Jelle Kok       Initial version created
</pre>
*/

#include "Player.h"
#include "ArsToolbox.h"
#include <stdio.h>

//DRIBLOVANIE
#include <fstream>
using namespace std;
//koniec DRIBLOVANIE

//#define PRIHRAVKY_FUZZY

SoccerCommand Player::kickToScore(  ){
    SoccerCommand soc(CMD_ILLEGAL);
  	//////////////////////////////////////////////////////////////////////////
	// FUZZY REGULACIA	START												//
	//////////////////////////////////////////////////////////////////////////

	double goalWidth = SS->getGoalWidth();
	double a = goalWidth * 0.5;
	int samples = 100;

	const int IN_Y_GOALIE = 0;			// input  fuzzygroup (lingvisticka premenna)
	const int IN_ME_TO_GOALIE = 1;		// input  fuzzygroup (lingvisticka premenna)
	const int OUT_Y_DIRECTION = 0;		// output fuzzygroup (lingvisticka premenna)

	const int BRANKAR_VLAVO = 0;		// charakteristicka funkcia pre IN_Y_GOALIE
	const int BRANKAR_TROCHU_VLAVO = 1; // charakteristicka funkcia pre IN_Y_GOALIE
 	const int BRANKAR_TROCHU_VPRAVO = 2;// charakteristicka funkcia pre IN_Y_GOALIE
	const int BRANKAR_VPRAVO = 3;		// charakteristicka funkcia pre IN_Y_GOALIE

	const int NALAVO_OD_BRANKARA  = 0; 	// charakteristicka funkcia pre IN_ME_TO_GOALIE
	const int BRANKAR_PREDOMNOU   = 1; 	// charakteristicka funkcia pre IN_ME_TO_GOALIE
	const int NAPRAVO_OD_BRANKARA = 2;	// charakteristicka funkcia pre IN_ME_TO_GOALIE

	const int KOPNI_EX_VLAVO = 0;		// charakteristicka funkcia pre OUT_Y_DIRECTION
	const int KOPNI_VLAVO = 1;			// charakteristicka funkcia pre OUT_Y_DIRECTION
	const int KOPNI_VPRAVO = 2;			// charakteristicka funkcia pre OUT_Y_DIRECTION
	const int KOPNI_EX_VPRAVO = 3;		// charakteristicka funckia pre OUT_Y_DIRECTION

	cFuzzyObj * fz = new cFuzzyObj();
 	fz->SetSamples(samples);

	fz->AddInput(-a, a);	// vytvorenie IN_Y_GOALIE
 	fz->AddInput(-a, a);	// vytovrenie IN_ME_TO_GOALIE
  	fz->AddOutput(-a, a);	// vytvorenie OUT_Y_DIRECTION

  	fz->AddInputMF(IN_Y_GOALIE, FZ_LEFT,      -0.75*a,  -0.4*a);					// BRANKAR_VLAVO
 	fz->AddInputMF(IN_Y_GOALIE, FZ_TRAPEZOID,  -0.6*a,  -0.3*a, -0.1*a, 0.1*a); 	// BRANKAR_TROCHU_VLAVO
   	fz->AddInputMF(IN_Y_GOALIE, FZ_TRAPEZOID,  -0.1*a,   0.1*a,  0.3*a, 0.6*a);		// BRANKAR_TROCHU_VPRAVO
 	fz->AddInputMF(IN_Y_GOALIE, FZ_RIGHT, 	    0.4*a,   0.75*a);					// BRANKAR_VPRAVO

	fz->AddInputMF(IN_ME_TO_GOALIE, FZ_LEFT,   	  -0.9*a, -0.2*a);					// NALAVO_OD_BRANKARA
	fz->AddInputMF(IN_ME_TO_GOALIE, FZ_TRAPEZOID, -0.3*a, -0.2*a, 0.2*a, 0.3*a);	// BRANKAR_PREDOMNOU
	fz->AddInputMF(IN_ME_TO_GOALIE, FZ_RIGHT,      0.2*a,  0.9*a);					// NAPRAVO_OD_BRANKARA


	fz->AddOutputMF(OUT_Y_DIRECTION, FZ_TRIANGLE,      -a, -0.9*a, -0.8*a);			// KOPNI_EX_VLAVO
	fz->AddOutputMF(OUT_Y_DIRECTION, FZ_TRAPEZOID, -0.9*a, -0.8*a, -0.6*a, -0.4*a);	// KOPNI_VLAVO
	fz->AddOutputMF(OUT_Y_DIRECTION, FZ_TRAPEZOID,  0.4*a,  0.6*a,  0.8*a,  0.9*a); // KOPNI_VPRAVO
	fz->AddOutputMF(OUT_Y_DIRECTION, FZ_TRIANGLE,   0.8*a,  0.9*a, a);				// KOPNI_EX_VPRAVO

	fz->AddRule(OUT_Y_DIRECTION, BRANKAR_VLAVO, NALAVO_OD_BRANKARA , KOPNI_EX_VLAVO);
	fz->AddRule(OUT_Y_DIRECTION, BRANKAR_VLAVO, BRANKAR_PREDOMNOU  , KOPNI_VPRAVO);
	fz->AddRule(OUT_Y_DIRECTION, BRANKAR_VLAVO, NAPRAVO_OD_BRANKARA, KOPNI_VPRAVO);

	fz->AddRule(OUT_Y_DIRECTION, BRANKAR_TROCHU_VLAVO, NALAVO_OD_BRANKARA  , KOPNI_EX_VLAVO);
	fz->AddRule(OUT_Y_DIRECTION, BRANKAR_TROCHU_VLAVO, BRANKAR_PREDOMNOU   , KOPNI_EX_VPRAVO);
	fz->AddRule(OUT_Y_DIRECTION, BRANKAR_TROCHU_VLAVO, NAPRAVO_OD_BRANKARA , KOPNI_VPRAVO);

  	fz->AddRule(OUT_Y_DIRECTION, BRANKAR_TROCHU_VPRAVO, NALAVO_OD_BRANKARA  , KOPNI_VLAVO);
 	fz->AddRule(OUT_Y_DIRECTION, BRANKAR_TROCHU_VPRAVO, BRANKAR_PREDOMNOU   , KOPNI_EX_VLAVO);
 	fz->AddRule(OUT_Y_DIRECTION, BRANKAR_TROCHU_VPRAVO, NAPRAVO_OD_BRANKARA , KOPNI_EX_VPRAVO);

  	fz->AddRule(OUT_Y_DIRECTION, BRANKAR_VPRAVO, NALAVO_OD_BRANKARA  , KOPNI_VLAVO);
  	fz->AddRule(OUT_Y_DIRECTION, BRANKAR_VPRAVO, BRANKAR_PREDOMNOU   , KOPNI_VLAVO);
  	fz->AddRule(OUT_Y_DIRECTION, BRANKAR_VPRAVO, NAPRAVO_OD_BRANKARA , KOPNI_EX_VPRAVO);


	VecPosition gp = WM->getGlobalPosition(OBJECT_OPPONENT_GOALIE);
	double opp_goalie_position = gp.getY();
 	if(opp_goalie_position < -a) {
  		opp_goalie_position = -a;
  	}
  	if(opp_goalie_position > a) {
   		opp_goalie_position = a;
   	}

 	VecPosition myPos = WM->getAgentGlobalPosition();
	VecPosition diff  = myPos - gp;
  	double my_rel_pos     = diff.getY();

	if(my_rel_pos<0) {
		if(my_rel_pos < -a) {
			my_rel_pos = -a;
 		}
 	}
 	else {
	if(my_rel_pos > a) {
		my_rel_pos = a;
	}
	}

 	double y_direction = fz->CountResult(OUT_Y_DIRECTION, DFZ_CENTROID, OPS_ZADEH, opp_goalie_position, my_rel_pos);
 	delete fz;

 	//////////////////////////////////////////////////////////////////////////
 	// FUZZY REGULACIA	END													//
  	//////////////////////////////////////////////////////////////////////////

	VecPosition posGoal( PITCH_LENGTH/2.0, y_direction );
	soc = kickTo( posGoal, SS->getBallSpeedMax() ); // kick maximal

  	ACT->putCommandInQueue( soc );
  	ACT->putCommandInQueue( turnNeckToObject( OBJECT_BALL, soc ) );
  	Log.log( 100, "kick ball" );
  	return soc;
}

SoccerCommand Player::kickToDrible(  ){
    SoccerCommand soc(CMD_ILLEGAL);
  	printf("Kicking to drible\n");
	// TODO: find better position to kick ball to, now random
  	VecPosition posGoal( PITCH_LENGTH/2.0,
          (-1 + 2*(WM->getCurrentCycle()%2)) * 0.4 * SS->getGoalWidth() );
  	soc = kickTo( posGoal, SS->getBallSpeedMax() / 30 ); // kick maximal

  	ACT->putCommandInQueue( soc );
  	ACT->putCommandInQueue( turnNeckToObject( OBJECT_BALL, soc ) );
  	Log.log( 100, "kick ball" );
}

void Player::findPosition(double dConfThr){
	
  struct circle {
  	double x,y,r;
  	bool tm;
  };
  circle circles[22];
  int circlesC = 0;

  circle lines[20];
  
  VecPosition vec1 = WM->getAgentGlobalPosition();
  double dDist;
  VecPosition posObj;
  int iIndex;
  
  for( ObjectT o = WM->iterateObjectStart( iIndex, OBJECT_SET_TEAMMATES_NO_GOALIE, dConfThr );
       o != OBJECT_ILLEGAL;
       o = WM->iterateObjectNext ( iIndex, OBJECT_SET_TEAMMATES_NO_GOALIE, dConfThr ) )
  {
    posObj    = WM->getGlobalPosition( o );
    dDist     = posObj.getDistanceTo( vec1 );
    
    circles[circlesC].x = posObj.getX();
    circles[circlesC].y = posObj.getY();    
    circles[circlesC].r = dDist / 5;
    circles[circlesC].tm = true;
    log->addObject(LOG_PLAYER_TEAM, posObj.getX(),posObj.getY());
    circlesC++;      
  }
  WM->iterateObjectDone( iIndex );
  
  for( ObjectT o = WM->iterateObjectStart( iIndex, OBJECT_SET_OPPONENTS, dConfThr );
       o != OBJECT_ILLEGAL;
       o = WM->iterateObjectNext ( iIndex, OBJECT_SET_OPPONENTS, dConfThr ) )
  {
    posObj    = WM->getGlobalPosition( o );
    dDist     = posObj.getDistanceTo( vec1 );
    
    circles[circlesC].x = posObj.getX();
    circles[circlesC].y = posObj.getY();    
    circles[circlesC].r = dDist / 5;
    circles[circlesC].tm = false;        
    log->addObject(LOG_PLAYER_OTHER, posObj.getX(),posObj.getY());
    circlesC++;      
  }
  
  // pre kazdy luc prejdeme kruznice
  double p1x, p1y, p2x, p2y;
  for (int i = 0; i < 20; i++) {
  	lines[i].r = 10000;
  	for (int j = 0; j < circlesC; j++) {
/*      if (circleLineIntersection(
            vec1.getX(), 
            vec1.getY(), 
            ?, 
            ?, 
            circle[j].getX(), 
            circle[j].getY(), 
            &p1x, 
            &p1y, 
            &p2x, 
            &p2y
      ) != 0) {
          
      }*/ 
  	}
  }
  
  WM->iterateObjectDone( iIndex );

  log->addObject(LOG_PLAYER_ACTIVE, vec1.getX(), vec1.getY());    
  log->addObject(LOG_BALL, 0, 0);
}

SoccerCommand Player::kickToPass(  ){
    SoccerCommand soc(CMD_ILLEGAL);
  	double dis;
  	ObjectT obj;

    log->situationBegin("Prihravka", WM->getCurrentCycle());
  
	// skusime najst lepsie miesto na strielanie
	findPosition(0.97);	
  	
  	// skusime najst hraca, ktory je najblizsie ku brane
  	obj = WM->getClosestInSetTo( OBJECT_SET_TEAMMATES_NO_GOALIE, goalMiddle,  &dis, 0.97, 1.0);

	// ak taky nie je, tak nahrame najblizsiemu
	if (obj == OBJECT_ILLEGAL)
		obj = WM->getClosestRelativeInSet(OBJECT_SET_TEAMMATES_NO_GOALIE, &dis, 1.0);
		
	// ak napriek tomu nemame ziadneho hraca, tak kopeme na branku  	
	if (obj == OBJECT_ILLEGAL){
		kickToScore();
	} else {
		// kolko trva lopte kym dojde ku hracovi?
		int cycles = WM->predictNrCyclesForDistance(OBJECT_BALL, dis, SS->getBallSpeedMax());		
		VecPosition vec1 = WM->getGlobalPosition(obj); 
		
		// ideme hracovi kopnut do behu s tym, ze uvazujeme ze ma 50% dash, getDashPowerRate sa nepouziva...
		VecPosition vec2 = WM->predictPosAfterNrCycles(obj, cycles, 50, NULL, NULL);		 
		VecPosition delta = vec2 - vec1;
		
		// urobime priemer, pretoze hraci nestihaju reagovat
		VecPosition vec3 = VecPosition(
            (vec2.getX() + cycles*vec1.getX()) / (cycles + 1), 
            (vec2.getY() + cycles*vec1.getY()) / (cycles + 1)
        );  
        
        double speed;		 

#ifdef PRIHRAVKY_FUZZY
	  	/////////////////////////////////////////////////////////////////////////////////////
	  	// FUZZY zistenie sily a koeficientu "do behu" - START
	  	/////////////////////////////////////////////////////////////////////////////////////
	  	int samples = 100;
	  	const double P3 = PITCH_LENGTH/3.0;
	  	const double a = 12;
	  	const double b = 1;
	  	
	  	const int IN_PLAYER_SPEED    = 0;	// input fuzzygroup, rychlost hraca
	  	const int IN_PLAYER_DISTANCE = 1;	// input fuzzygroup, vzdialenost hraca
	  	const int OUT_BALL_SPEED 	 = 0;	// output fuzzygroup, rychlost kopu do lopty
	  	const int OUT_BALL_DIRECTION = 1; 	// output fuzzygroup, velkost prekopavanej nahravky
	  	
	  	const int PLAYER_SLOW 	= 0;		// charakteristicka funkcia pre IN_PLAYER_SPEED
	  	const int PLAYER_FAST		= 1;	// charakteristicka funkcia pre IN_PLAYER_SPEED
	  	const int PLAYER_VERY_FAST = 2;		// charakteristicka funkcia pre IN_PLAYER_SPEED
	  	
	  	const int PLAYER_NEAR		= 0;	// charakteristicka funkcia pre IN_PLAYER_DISTANCE
	  	const int PLAYER_MIDDLE	= 1;		// charakteristicka funkcia pre IN_PLAYER_DISTANCE
	  	const int PLAYER_FAR		= 2;	// charakteristicka funkcia pre IN_PLAYER_DISTNACE
	  	
	  	const int POWER_SMALL	= 0;		// charakteristicka funkcia pre OUT_BALL_SPEED
	  	const int POWER_MEAN	= 1;		// charakteristicka funkcia pre OUT_BALL_SPEED
	  	const int POWER_BIG		= 2;		// charakteristicka funkcia pre OUT_BALL_SPEED
	  	const int POWER_MAX		= 3;		// charakteristicka funkcia pre OUT_BALL_SPEED
	  	
	  	const int BALL_NEAR		= 0;		// charakteristicka funkcia pre OUT_BALL_DIRECTION
	  	const int BALL_MIDDLE	= 1;		// charakteristicka funkcia pre OUT_BALL_DIRECTION
	  	const int BALL_FAR		= 2;		// charakteristicka funkcia pre OUT_BALL_DIRECTION
	  	
	  	cFuzzyObj * silaFuzzy = new cFuzzyObj();
	  	silaFuzzy->SetSamples(samples);
	  	
	  	silaFuzzy->AddInput(0, a);			// interval pre IN_PLAYER_SPEED
	  	silaFuzzy->AddInput(0, P3);			// interval pre IN_PLAYER_DISTANCE
	  	silaFuzzy->AddOutput(0, b);			// interval pre OUT_BALL_SPEED
	  	silaFuzzy->AddOutput(0, b);			// interval pre OUT_BALL_DIRECTION
	  	
	  	
	  	silaFuzzy->AddInputMF(IN_PLAYER_SPEED, FZ_LEFT, 0, 0.33*a);							// PLAYER_SLOWLY
	  	silaFuzzy->AddInputMF(IN_PLAYER_SPEED, FZ_TRAPEZOID, 0.177*a, 0.33*a, 0.66*a, 0.833*a);	// PLAYER_FAST
	  	silaFuzzy->AddInputMF(IN_PLAYER_SPEED, FZ_RIGHT, 0.66*a, a);						// PLAYER_VERY_FAST
	  	
	  	
	  	silaFuzzy->AddInputMF(IN_PLAYER_DISTANCE, FZ_LEFT, 0.125*P3,  0.5*P3);							// PLAYER_NEAR
	  	silaFuzzy->AddInputMF(IN_PLAYER_DISTANCE, FZ_TRAPEZOID, 0.125*P3, 0.375*P3, 0.625*P3, 0.875*P3);// PLAYER_MIDDLE
	  	silaFuzzy->AddInputMF(IN_PLAYER_DISTANCE, FZ_RIGHT, 0.5*P3, 0.875*P3);							// PLAYER_FAR
	  	
	  	silaFuzzy->AddOutputMF(OUT_BALL_SPEED, FZ_LEFT, 0.125*b, 0.375*b);						// POWER_SMALL
	  	silaFuzzy->AddOutputMF(OUT_BALL_SPEED, FZ_TRAPEZOID, 0.125*b, 0.375*b, 0.5*b, 0.625*b);	// POWER_MEAN
	  	silaFuzzy->AddOutputMF(OUT_BALL_SPEED, FZ_TRAPEZOID, 0.375*b, 0.5*b, 0.625*b, 0.875*b); // POWER_BIG
	  	silaFuzzy->AddOutputMF(OUT_BALL_SPEED, FZ_RIGHT, 0.625*b, 0.875*b);						// POWER_MAX
	  	
	  	silaFuzzy->AddOutputMF(OUT_BALL_DIRECTION, FZ_LEFT, 0.125*b, 0.375*b);						 // BALL_NEAR
	  	silaFuzzy->AddOutputMF(OUT_BALL_DIRECTION, FZ_TRAPEZOID, 0.125*b, 0.375*b, 0.625*b, 0.875*b);// BALL_MIDLLE
	  	silaFuzzy->AddOutputMF(OUT_BALL_DIRECTION, FZ_RIGHT, 0.625*b, 0.875*b);						 // BALL_FAR
	  	
	  	// Prva tabulka
	  	
	  	silaFuzzy->AddRule(OUT_BALL_SPEED, PLAYER_SLOW, PLAYER_NEAR, POWER_SMALL );
	  	silaFuzzy->AddRule(OUT_BALL_SPEED, PLAYER_SLOW, PLAYER_MIDDLE, POWER_MEAN);
	  	silaFuzzy->AddRule(OUT_BALL_SPEED, PLAYER_SLOW,	PLAYER_FAR, POWER_BIG);
	  	
	  	silaFuzzy->AddRule(OUT_BALL_SPEED, PLAYER_FAST, PLAYER_NEAR, POWER_MEAN);
	  	silaFuzzy->AddRule(OUT_BALL_SPEED, PLAYER_FAST, PLAYER_MIDDLE, POWER_MEAN);
	  	silaFuzzy->AddRule(OUT_BALL_SPEED, PLAYER_FAST, PLAYER_FAR, POWER_BIG);
	  	
	  	silaFuzzy->AddRule(OUT_BALL_SPEED, PLAYER_VERY_FAST, PLAYER_NEAR, POWER_BIG); 
	  	silaFuzzy->AddRule(OUT_BALL_SPEED, PLAYER_VERY_FAST, PLAYER_MIDDLE, POWER_BIG);
	  	silaFuzzy->AddRule(OUT_BALL_SPEED, PLAYER_VERY_FAST, PLAYER_FAR, POWER_MAX);
	  	
	  	// Druha tabulka
	  	
	  	silaFuzzy->AddRule(OUT_BALL_DIRECTION, PLAYER_SLOW, PLAYER_NEAR, BALL_NEAR );
	  	silaFuzzy->AddRule(OUT_BALL_DIRECTION, PLAYER_SLOW, PLAYER_MIDDLE, BALL_MIDDLE);
	  	silaFuzzy->AddRule(OUT_BALL_DIRECTION, PLAYER_SLOW,	PLAYER_FAR, BALL_FAR);
	  	
	  	silaFuzzy->AddRule(OUT_BALL_DIRECTION, PLAYER_FAST, PLAYER_NEAR, BALL_MIDDLE);
	  	silaFuzzy->AddRule(OUT_BALL_DIRECTION, PLAYER_FAST, PLAYER_MIDDLE, BALL_MIDDLE);
	  	silaFuzzy->AddRule(OUT_BALL_DIRECTION, PLAYER_FAST, PLAYER_FAR, BALL_FAR);
	  	
	  	silaFuzzy->AddRule(OUT_BALL_DIRECTION, PLAYER_VERY_FAST, PLAYER_NEAR, BALL_FAR); 
	  	silaFuzzy->AddRule(OUT_BALL_DIRECTION, PLAYER_VERY_FAST, PLAYER_MIDDLE, BALL_FAR);
	  	silaFuzzy->AddRule(OUT_BALL_DIRECTION, PLAYER_VERY_FAST, PLAYER_FAR, BALL_FAR);
	  	
	  	// rychlost hraca
	  	double player_speed = vec2.getMagnitude();
	  	// ak je jeho "rychlost" vacsia, ako nas rozzsah, tak ju skreseme
	  	if(player_speed > a) {
	  		player_speed = a;
	  	}
	  	
	  	// ak je ich vzdialenost vecsia, ako nas rozsah, tak ju skreseme
	  	double dist = dis;
	  	if(dis > P3) {
	  		dist = P3;
	  	}
	  	
	  	double speed_coeficient = silaFuzzy->CountResult(OUT_BALL_SPEED, DFZ_CENTROID, OPS_ZADEH, player_speed, dist);
	  	double run_coefficient   = silaFuzzy->CountResult(OUT_BALL_DIRECTION, DFZ_CENTROID, OPS_ZADEH, player_speed, dist);
	  	
	  	delete silaFuzzy;
	  	
	  	/////////////////////////////////////////////////////////////////////////////////////
	  	// FUZZY zistenie sily a koeficientu "do behu" - END
	  	/////////////////////////////////////////////////////////////////////////////////////
	  	
		speed = SS->getBallSpeedMax()*speed_coeficient;
		VecPosition vec4 = vec2 + (delta * run_coefficient);
        log->addObject(LOG_BALL, vec4.getX(),vec4.getY());
	  	soc = kickTo( vec4, speed );
	  	
#else
	  	
	  	// zistime silu a kopneme
		speed = SS->getBallSpeedMax() * ((dis + delta.getMagnitude()) / (PITCH_LENGTH / (double)3));
		soc = kickTo( vec3, speed );
        log->addObject(LOG_BALL, vec3.getX(),vec3.getY());
        
#endif
	
	  	ACT->putCommandInQueue( soc );
	  	ACT->putCommandInQueue( turnNeckToObject( OBJECT_BALL, soc ) );
	}
    log->situationEnd();
    return soc;
}
/*!This method is the first complete simple team and defines the actions taken
   by all the players on the field (excluding the goalie). It is based on the
   high-level actions taken by the simple team FC Portugal that it released in
   2000. The players do the following:
   - if ball is kickable
       kick ball to goal (random corner of goal)
   - else if i am fastest player to ball 
       intercept the ball
   - else
       move to strategic position based on your home position and pos ball */
SoccerCommand Player::deMeer5(  )
{

  SoccerCommand soc(CMD_ILLEGAL);
  VecPosition   posAgent = WM->getAgentGlobalPosition();
  VecPosition   posBall  = WM->getBallPos();
  int           iTmp;
  
  if( WM->getCurrentCycle() == 1 )
  	cycle = -20;

  if( WM->isBeforeKickOff( ) )
  {
    if( WM->isKickOffUs( ) && WM->getPlayerNumber() == 9 ) // 9 takes kick
    {
      if( WM->isBallKickable() )
      {
      	
      	
        VecPosition posGoal( PITCH_LENGTH/2.0,
                             (-1 + 2*(WM->getCurrentCycle()%2)) * 
                             0.4 * SS->getGoalWidth() );
                            
        soc = kickTo( posGoal, SS->getBallSpeedMax() ); // kick maximal
        Log.log( 100, "take kick off" );        
      }
      else
      {
        soc = intercept( false );  
        Log.log( 100, "move to ball to take kick-off" );
      }  
      ACT->putCommandInQueue( soc );
      ACT->putCommandInQueue( turnNeckToObject( OBJECT_BALL, soc ) );
      return soc;
    }  
    if( formations->getFormation() != FT_INITIAL || // not in kickoff formation
        posAgent.getDistanceTo( WM->getStrategicPosition() ) > 2.0 )  
    {
      formations->setFormation( FT_INITIAL );       // go to kick_off formation
      ACT->putCommandInQueue( soc=teleportToPos( WM->getStrategicPosition() ));
    }
    else                                            // else turn to center
    {
      ACT->putCommandInQueue( soc=turnBodyToPoint( VecPosition( 0, 0 ), 0 ) );
      ACT->putCommandInQueue( alignNeckWithBody( ) );
    }
  }
  else
  {
    formations->setFormation( FT_433_OFFENSIVE );
    soc.commandType = CMD_ILLEGAL;

    if( WM->getConfidence( OBJECT_BALL ) < PS->getBallConfThr() )
    {
      ACT->putCommandInQueue( soc = searchBall() );   // if ball pos unknown
      ACT->putCommandInQueue( alignNeckWithBody( ) ); // search for it
    }
    else if( WM->isBallKickable())                    // if kickable
    {
    	// DRIBLOVANIE
      	
      	double angle;
      	
      	soc = dribble( 0.0 , DRIBBLE_SLOW);
      	
      	//SITUATIONSLOG
      	
      	log->situationBegin("Driblovanie", WM->getCurrentCycle());

        if( fabs( WM->getCurrentCycle() - cycle ) > 12 )
        {
  	
#ifdef DEBUG
	LOG_PRINTP(LOG_SOMI, "\nnew dribble: %d", cycle);
#endif //DEBUG

        }
        else
        {

#ifdef DEBUG
	LOG_PRINTP(LOG_SOMI, "\ndribble again: %d", cycle);
#endif //DEBUG

        }
        cycle = WM->getCurrentCycle();
               
        VecPosition vec = WM->getAgentGlobalPosition();
        double dDist;
        VecPosition posObj;
        int iIndex;
  
        for( ObjectT o = WM->iterateObjectStart( iIndex, 
                         OBJECT_SET_TEAMMATES_NO_GOALIE );
             o != OBJECT_ILLEGAL;
             o = WM->iterateObjectNext ( iIndex, OBJECT_SET_TEAMMATES_NO_GOALIE ) )
        {
          posObj    = WM->getGlobalPosition( o );
          dDist     = posObj.getDistanceTo( vec );
    
          if (dDist <= 20.0)
            log->addObject(LOG_PLAYER_TEAM, posObj.getX(),posObj.getY());
        } 
        
        for( ObjectT o = WM->iterateObjectStart( iIndex, OBJECT_SET_OPPONENTS );
             o != OBJECT_ILLEGAL;
             o = WM->iterateObjectNext ( iIndex, OBJECT_SET_OPPONENTS ) )
        {
          posObj    = WM->getGlobalPosition( o );
          dDist     = posObj.getDistanceTo( vec );
    
          if (dDist <= 20.0)
          	log->addObject(LOG_PLAYER_OTHER, posObj.getX(),posObj.getY());
        }
        
        log->addObject( LOG_PLAYER_ACTIVE, WM->getAgentGlobalPosition().getX(), 
                        WM->getAgentGlobalPosition().getY() );
        log->addObject( LOG_CIRCLE, WM->getBallPos().getX(), 
                        WM->getBallPos().getY(), 10.0 );
        log->addObject( LOG_BALL, WM->getBallPos().getX(), WM->getBallPos().getY() );
        
        log->situationEnd();
		
        //koniec SITUATIONSLOG
      	
      	ACT->putCommandInQueue( soc );
      	ACT->putCommandInQueue( turnNeckToObject( OBJECT_BALL, soc ) );      	   
      	
      	// KONIEC DRIBLOVANIE
    }
    else if( WM->getFastestInSetTo( OBJECT_SET_TEAMMATES, OBJECT_BALL, &iTmp )
              == WM->getAgentObjectType()  && !WM->isDeadBallThem() )
    {                                                // if fastest to ball
      Log.log( 100, "I am fastest to ball; can get there in %d cycles", iTmp );
      soc = intercept( false );                      // intercept the ball

      if( soc.commandType == CMD_DASH &&             // if stamina low
          WM->getAgentStamina().getStamina() <
             SS->getRecoverDecThr()*SS->getStaminaMax()+200 )
      {
        soc.dPower = 30.0 * WM->getAgentStamina().getRecovery(); // dash slow
        ACT->putCommandInQueue( soc );
        ACT->putCommandInQueue( turnNeckToObject( OBJECT_BALL, soc ) );
      }
      else                                           // if stamina high
      {
        ACT->putCommandInQueue( soc );               // dash as intended
        ACT->putCommandInQueue( turnNeckToObject( OBJECT_BALL, soc ) );
      }
     }
     else if( posAgent.getDistanceTo(WM->getStrategicPosition()) >
                  1.5 + fabs(posAgent.getX()-posBall.getX())/10.0)
                                                  // if not near strategic pos
     {
       if( WM->getAgentStamina().getStamina() >     // if stamina high
                            SS->getRecoverDecThr()*SS->getStaminaMax()+800 )
       {
         soc = moveToPos(WM->getStrategicPosition(),
                         PS->getPlayerWhenToTurnAngle());
         ACT->putCommandInQueue( soc );            // move to strategic pos
         ACT->putCommandInQueue( turnNeckToObject( OBJECT_BALL, soc ) );
       }
       else                                        // else watch ball
       {
         ACT->putCommandInQueue( soc = turnBodyToObject( OBJECT_BALL ) );
         ACT->putCommandInQueue( turnNeckToObject( OBJECT_BALL, soc ) );
       }
     }
     else if( fabs( WM->getRelativeAngle( OBJECT_BALL ) ) > 1.0 ) // watch ball
     {
       ACT->putCommandInQueue( soc = turnBodyToObject( OBJECT_BALL ) );
       ACT->putCommandInQueue( turnNeckToObject( OBJECT_BALL, soc ) );
     }
     else                                         // nothing to do
       ACT->putCommandInQueue( SoccerCommand(CMD_TURNNECK,0.0) );
   }
  return soc;
}

/*!This method is a simple goalie based on the goalie of the simple Team of
   FC Portugal. It defines a rectangle in its penalty area and moves to the
   position on this rectangle where the ball intersects if you make a line
   between the ball position and the center of the goal. If the ball can
   be intercepted in the own penalty area the ball is intercepted and catched.
*/
SoccerCommand Player::deMeer5_goalie(  )
{
  int i;
  SoccerCommand soc;
  VecPosition   posAgent = WM->getAgentGlobalPosition();
  AngDeg        angBody  = WM->getAgentGlobalBodyAngle();

  // define the top and bottom position of a rectangle in which keeper moves
  static const VecPosition posLeftTop( -PITCH_LENGTH/2.0 +
               0.7*PENALTY_AREA_LENGTH, -PENALTY_AREA_WIDTH/4.0 );
  static const VecPosition posRightTop( -PITCH_LENGTH/2.0 +
               0.7*PENALTY_AREA_LENGTH, +PENALTY_AREA_WIDTH/4.0 );

  // define the borders of this rectangle using the two points.
  static Line  lineFront = Line::makeLineFromTwoPoints(posLeftTop,posRightTop);
  static Line  lineLeft  = Line::makeLineFromTwoPoints(
                         VecPosition( -50.0, posLeftTop.getY()), posLeftTop );
  static Line  lineRight = Line::makeLineFromTwoPoints(
                         VecPosition( -50.0, posRightTop.getY()),posRightTop );


  if( WM->isBeforeKickOff( ) )
  {
    if( formations->getFormation() != FT_INITIAL || // not in kickoff formation
        posAgent.getDistanceTo( WM->getStrategicPosition() ) > 2.0 )  
    {
      formations->setFormation( FT_INITIAL );       // go to kick_off formation
      ACT->putCommandInQueue( soc=teleportToPos(WM->getStrategicPosition()) );
    }
    else                                            // else turn to center
    {
      ACT->putCommandInQueue( soc = turnBodyToPoint( VecPosition( 0, 0 ), 0 ));
      ACT->putCommandInQueue( alignNeckWithBody( ) );
    }
    return soc;
  }

  if( WM->getConfidence( OBJECT_BALL ) < PS->getBallConfThr() )
  {                                                // confidence ball too  low
    ACT->putCommandInQueue( searchBall() );        // search ball
    ACT->putCommandInQueue( alignNeckWithBody( ) );
  }
  else if( WM->getPlayMode() == PM_PLAY_ON || WM->isFreeKickThem() ||
           WM->isCornerKickThem() )               
  {
    if( WM->isBallCatchable() )
    {
      ACT->putCommandInQueue( soc = catchBall() );
      ACT->putCommandInQueue( turnNeckToObject( OBJECT_BALL, soc ) );
    }
     else if( WM->isBallKickable() )
    {
       soc = kickTo( VecPosition(0,posAgent.getY()*2.0), 2.0 );    
       ACT->putCommandInQueue( soc );
       ACT->putCommandInQueue( turnNeckToObject( OBJECT_BALL, soc ) );
    }
    else if( WM->isInOwnPenaltyArea( getInterceptionPointBall( &i, true ) ) &&
             WM->getFastestInSetTo( OBJECT_SET_PLAYERS, OBJECT_BALL, &i ) == 
                                               WM->getAgentObjectType() )
    {
      ACT->putCommandInQueue( soc = intercept( true ) );
      ACT->putCommandInQueue( turnNeckToObject( OBJECT_BALL, soc ) );
    }
    else
    {
      // make line between own goal and the ball
      VecPosition posMyGoal = ( WM->getSide() == SIDE_LEFT )
             ? SoccerTypes::getGlobalPositionFlag(OBJECT_GOAL_L, SIDE_LEFT )
             : SoccerTypes::getGlobalPositionFlag(OBJECT_GOAL_R, SIDE_RIGHT);
      Line lineBall = Line::makeLineFromTwoPoints( WM->getBallPos(),posMyGoal);

      // determine where your front line intersects with the line from ball
      VecPosition posIntersect = lineFront.getIntersection( lineBall );

      // outside rectangle, use line at side to get intersection
      if (posIntersect.isRightOf( posRightTop ) )
        posIntersect = lineRight.getIntersection( lineBall );
      else if (posIntersect.isLeftOf( posLeftTop )  )
        posIntersect = lineLeft.getIntersection( lineBall );

      if( posIntersect.getX() < -49.0 )
        posIntersect.setX( -49.0 );
        
      // and move to this position
      if( posIntersect.getDistanceTo( WM->getAgentGlobalPosition() ) > 0.5 )
      {
        soc = moveToPos( posIntersect, PS->getPlayerWhenToTurnAngle() );
        ACT->putCommandInQueue( soc );
        ACT->putCommandInQueue( turnNeckToObject( OBJECT_BALL, soc ) );
      }
      else
      {
        ACT->putCommandInQueue( soc = turnBodyToObject( OBJECT_BALL ) );
        ACT->putCommandInQueue( turnNeckToObject( OBJECT_BALL, soc ) );
      }
    }
  }
  else if( WM->isFreeKickUs() == true || WM->isGoalKickUs() == true )
  {
    if( WM->isBallKickable() )
    {
      if( WM->getTimeSinceLastCatch() == 25 && WM->isFreeKickUs() )
      {
        // move to position with lesser opponents.
        if( WM->getNrInSetInCircle( OBJECT_SET_OPPONENTS, 
                                          Circle(posRightTop, 15.0 )) <
            WM->getNrInSetInCircle( OBJECT_SET_OPPONENTS, 
                                           Circle(posLeftTop,  15.0 )) )
          soc.makeCommand( CMD_MOVE,posRightTop.getX(),posRightTop.getY(),0.0);
        else
          soc.makeCommand( CMD_MOVE,posLeftTop.getX(), posLeftTop.getY(), 0.0);
        ACT->putCommandInQueue( soc );
      }
      else if( WM->getTimeSinceLastCatch() > 28 )
      {
        soc = kickTo( VecPosition(0,posAgent.getY()*2.0), 2.0 );    
        ACT->putCommandInQueue( soc );
      }
      else if( WM->getTimeSinceLastCatch() < 25 )
      {
        VecPosition posSide( 0.0, posAgent.getY() ); 
        if( fabs( (posSide - posAgent).getDirection() - angBody) > 10 )
        {
          soc = turnBodyToPoint( posSide );
          ACT->putCommandInQueue( soc );
        }
        ACT->putCommandInQueue( turnNeckToObject( OBJECT_BALL, soc ) );
      }
    }
    else if( WM->isGoalKickUs()  )
    {
      ACT->putCommandInQueue( soc = intercept( true ) );
      ACT->putCommandInQueue( turnNeckToObject( OBJECT_BALL, soc ) );
    }
    else
      ACT->putCommandInQueue( turnNeckToObject( OBJECT_BALL, soc ) );
  }
  else
  {
     ACT->putCommandInQueue( soc = turnBodyToObject( OBJECT_BALL ) );
     ACT->putCommandInQueue( turnNeckToObject( OBJECT_BALL, soc ) );
  }
  return soc;
} 